﻿using Dynamicweb.Ecommerce.LiveIntegration.Logging;
using System;
using System.Globalization;

namespace Dynamicweb.Ecommerce.LiveIntegration.Licensing
{
    internal class VersionInfo
    {
        internal Version AppVersion { get; private set; }
        internal int CustomVersion { get; private set; }
        internal string ServerVersion { get; private set; }

        internal VersionInfo()
        {
            CustomVersion = -1;
        }

        internal static bool TryParse(string version, out VersionInfo result)
        {
            result = new VersionInfo();
            if (version == null)
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, "Version Error: String to be parsed is null.");
                return false;
            }

            string v = version.ToLower();
            if (v.IndexOf("_nav") > 0)
            {
                string[] parts = v.Split(new string[] { "_nav" }, StringSplitOptions.RemoveEmptyEntries);
                if (parts.Length > 0)
                {
                    result.ServerVersion = parts[1];
                }
                v = parts[0];
            }

            string[] parsedComponents = v.Split(new char[] { '.' });
            int parsedComponentsLength = parsedComponents.Length;
            if ((parsedComponentsLength < 2) || (parsedComponentsLength > 5))
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, $"Version Error: Bad number of components in '{version}'.");
                return false;
            }

            int major, minor, build, revision, customVersion;
            if (!TryParseComponent(parsedComponents[0], "major version", out major))
            {
                return false;
            }

            if (!TryParseComponent(parsedComponents[1], "minor version", out minor))
            {
                return false;
            }

            parsedComponentsLength -= 2;

            if (parsedComponentsLength > 0)
            {
                if (!TryParseComponent(parsedComponents[2], "build", out build))
                {
                    return false;
                }

                parsedComponentsLength--;

                if (parsedComponentsLength > 0)
                {
                    if (!TryParseComponent(parsedComponents[3], "revision", out revision))
                    {
                        return false;
                    }
                    else
                    {
                        result.AppVersion = new Version(major, minor, build, revision);
                    }
                }
                else
                {
                    result.AppVersion = new Version(major, minor, build);
                }

                parsedComponentsLength--;

                if (parsedComponentsLength > 0)
                {
                    if (!TryParseComponent(parsedComponents[4], "custom version", out customVersion))
                    {
                        return false;
                    }
                    else
                    {
                        result.CustomVersion = customVersion;
                    }
                }
            }
            else
            {
                result.AppVersion = new Version(major, minor);
            }

            return true;
        }

        private static bool TryParseComponent(string component, string componentName, out int parsedComponent)
        {
            if (!int.TryParse(component, NumberStyles.Integer, CultureInfo.InvariantCulture, out parsedComponent))
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, $"Version Error: Non-integer value '{component}' in '{componentName}'.");
                return false;
            }

            if (parsedComponent < 0)
            {
                Logger.Instance.Log(ErrorLevel.DebugInfo, $"Version Error: Number '{component}' is out of range in '{componentName}'.");
                return false;
            }

            return true;
        }
    }
}
